"use strict";
// App constructor
const App = function(canvas, overlay) {
  this.canvas = canvas;
  this.overlay = overlay;

  // if no GL support, cry
  this.gl = canvas.getContext("experimental-webgl");
  if (this.gl === null) {
    throw new Error("Browser does not support WebGL");

  }
  this.gl.pendingResources = {};
  // create a simple scene
  this.scene = new Scene(this.gl);

  this.resize();
  this.socket = io.connect('localhost');
  //this.socket = io.connect('http://cg.iit.bme.hu');
  this.socket.on('syncObjects', (data) => {
    this.scene.syncObjects(data.clientId, data.state);
  });
  this.socket.on('syncState', (data) => {
    this.scene.syncState(data.clientId, data.state);
  });
  this.frameCount = 0;
};

// match WebGL rendering resolution and viewport to the canvas size
App.prototype.resize = function() {
  this.canvas.width = this.canvas.clientWidth;
  this.canvas.height = this.canvas.clientHeight;
  this.gl.viewport(0, 0, this.canvas.width, this.canvas.height);
  this.scene.camera.setAspectRatio(this.canvas.clientWidth/this.canvas.clientHeight); 
};

App.prototype.registerEventHandlers = function() {
  document.onkeydown = (event) => {
    if(keyboardMap[event.keyCode] === 'W') {
      //LABTODO: message to server
      this.scene.setThrust(1);
    } else if(keyboardMap[event.keyCode] === 'S') {
      //LABTODO: message to server
      this.scene.setThrust(-1);
    } else if(keyboardMap[event.keyCode] === 'A') {
      //LABTODO: message to server
      this.scene.setTorque(1);
    } else if(keyboardMap[event.keyCode] === 'D') {
      //LABTODO: message to server
      this.scene.setTorque(-1);
    }
  };
  document.onkeyup = (event) => {
    if(keyboardMap[event.keyCode] === 'W') {
      //LABTODO: message to server
      this.scene.setThrust(0);
    } else if(keyboardMap[event.keyCode] === 'S') {
      //LABTODO: message to server
      this.scene.setThrust(0);
    } else if(keyboardMap[event.keyCode] === 'A') {
      //LABTODO: message to server
      this.scene.setTorque(0);
    } else if(keyboardMap[event.keyCode] === 'D') {
      //LABTODO: message to server
      this.scene.setTorque(0);
    }
  };
  this.canvas.onmousedown = (event) => {
    //jshint unused:false
  };
  this.canvas.onmousemove = (event) => {
    //jshint unused:false
    event.stopPropagation();
  };
  this.canvas.onmouseout = (event) => {
    //jshint unused:false
  };
  this.canvas.onmouseup = (event) => {
    //jshint unused:false
  };
  window.addEventListener('resize', () => {
    this.resize();
  });
  window.addEventListener('beforeunload', () => {
    // LABTODO: message to server
  }); 
  window.requestAnimationFrame(() => {
    this.update();
  });
};

// animation frame update
App.prototype.update = function() {

  const pendingResourceNames = Object.keys(this.gl.pendingResources);
  if (pendingResourceNames.length === 0) {
    // animate and draw scene
    this.scene.update(this.gl);
    this.overlay.innerHTML = "Ready.";
  } else {
    this.overlay.innerHTML = "Loading: " + pendingResourceNames;
  }
  
  if(this.frameCount % 20 === 0) {
    //LABTODO: message to server, request a state sync
  }
  this.frameCount++;

  // refresh
  window.requestAnimationFrame(() => {
    this.update();
  });
};

// entry point from HTML
window.addEventListener('load', function() {
  const canvas = document.getElementById("canvas");
  const overlay = document.getElementById("overlay");
  overlay.innerHTML = "WebGL";

  const app = new App(canvas, overlay);
  app.registerEventHandlers();
});

