"use strict";
const Material = function(gl, program) {
  this.gl = gl;
  this.program = program;
  Object.keys(program.uniforms).forEach((uniformName) => {
    const uniform = program.uniforms[uniformName];
    const reflectionVariable =
        UniformReflectionFactories.makeVar(gl, uniform.type, uniform.arraySize, uniform.textureUnit);
    Object.defineProperty(this, uniformName, {value: reflectionVariable} );
  });
  return new Proxy(this, {
    get : function(target, name){
      if(!(name in target)){
        console.error("WARNING: Ignoring attempt to access material property '" +
            name + "'. Is '" + name + "' an unused uniform?" );
        return Material.dummy;
      }
      return target[name];
    },
  });
};

Material.dummy = new Proxy(new Function(), {
  get: function(target, name){
    return Material.dummy;
  },
  apply: function(target, thisArg, args){
    return Material.dummy;
  },
});

Material.unused = new Proxy({}, {
  get: function(target, name){
    console.trace("WARNING: Attempt to set unused uniform ignored.");
    return Material.dummy;
  },
  apply: function(target, thisArg, args){
    return Material.dummy;
  },
});

Material.prototype.commit = function() {
  const gl = this.gl;
  this.program.commit();
  Object.keys(this.program.uniforms).forEach( (uniformName) => {
    const uniform = this.program.uniforms[uniformName];
    this[uniformName].commit(gl, uniform.location);
  });
};