"use strict";
const Scene = function(gl) {
  this.vsTrafo = new Shader(gl, gl.VERTEX_SHADER, "trafo_vs.essl");
  this.fsTextured = new Shader(gl, gl.FRAGMENT_SHADER, "textured_fs.essl");
  this.texturedProgram = new Program(gl, this.vsTrafo, this.fsTextured);
  this.quadGeometry = new QuadGeometry(gl);
  const addMesh = (textureFileName, animTileSize) => {
    const material = new Material(gl, this.texturedProgram);
    material.colorTexture.set( new Texture2D(gl, 'media/' + textureFileName + '.png') );
    material.texScale.set(animTileSize);
    this[textureFileName + 'Mesh'] = new Mesh(this.quadGeometry, material);
  };

  addMesh('raider', new Vec2(1, 1));
  addMesh('orb', new Vec2(0.2, 0.2));

  this.dynamicObjects = [];
  this.playerId = null;

  this.gameStatePool = new GameStatePool();
  this.gameState = new GameState(this.gameStatePool);

  //LABTODO: bízzuk ezt a szerverre
  this.playerId = 0;
  this.gameState.addPlayer(0);
  this.gameState.addObject(this.gameStatePool, 0, true, 'raider');
  this.dynamicObjects.push(new GameObject(this.raiderMesh));

  this.staticObjects = [];
  for(let i=0; i<16; i++) {
    for(let j=0; j<16; j++) {
      const orb = new GameObject(this.orbMesh);
      orb.position.set(i*8-4, j*8-4);
      orb.animFreq = 50;
      this.staticObjects.push(orb);
     }      
  }

  this.camera = new OrthoCamera();

  gl.enable(gl.BLEND);
  gl.blendFunc(
    gl.SRC_ALPHA,
    gl.ONE_MINUS_SRC_ALPHA);

  this.timeAtStart = new Date().getTime();
  this.timeAtLastFrame = 0.0;
};

Scene.prototype.setThrust = function(thrust) {
  this.gameState.thrusts.at(
    this.gameState.avatars[this.playerId].objectIndex
    ).set(thrust);
};

Scene.prototype.setTorque = function(torque) {
  this.gameState.torques.at(
    this.gameState.avatars[this.playerId].objectIndex
    ).set(torque);
};

Scene.prototype.update = function(gl, keysPressed) {
  //jshint bitwise:false
  //jshint unused:false
  if(this.playerId === null) {
    return;
  }

  const time = (new Date().getTime() - this.timeAtStart) / 1000.0;
  const dt = time - this.timeAtLastFrame;
  this.timeAtLastFrame = time;

  // clear the screen
  gl.clearColor(0.6, 0.0, 0.3, 1.0);
  gl.clearDepth(1.0);
  gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);

  this.gameState.update(dt);

  this.camera.position.set(this.gameState.positions.at(
    this.gameState.avatars[this.playerId].objectIndex));
  this.camera.updateViewProjMatrix();

  for(let i=0; i<this.dynamicObjects.length; i++) {
    this.dynamicObjects[i].position.set( this.gameState.positions.at(i) );
    this.dynamicObjects[i].orientation.set( this.gameState.orientations.at(i) );
    this.dynamicObjects[i].draw(this.camera, time);
  }

  for(let i=0; i<this.staticObjects.length; i++) {
    this.staticObjects[i].draw(this.camera, time);
  }
};

Scene.prototype.syncObjects = function(clientId, serverState) {
  this.dynamicObjects = serverState.dynamicObjects.map( (obj) => {
    return new GameObject( this[obj.meshName + 'Mesh'] );
  });
  this.gameState.dynamicObjects = serverState.dynamicObjects;
  this.gameState.resizeArrays(this.gameStatePool);
  this.syncState(clientId, serverState);
};

Scene.prototype.syncState = function(clientId, serverState) {
  if(clientId !== null) {
    this.playerId = clientId;
    console.log("I am " + this.playerId);
  }
  this.gameState.set(serverState);
};

